import os
import sys

import click

from ..custom_logging import error
from ..list.command import get_list


@click.command()
@click.option(
    "-id", "--id", default=None, help="Kill a given app ID", type=int
)
@click.option(
    "-a",
    "--all",
    is_flag=True,
    default=False,
    help="Kill all Streamlit apps",
    type=bool,
)
@click.pass_context
def kill(ctx: click.Context, id: int, all: bool):
    """🔫 Kill a given Streamlit app running locally!"""

    if (id is not None and all) or (id is None and not all):
        error(
            "You must either input the app process ID --id or choose --all to"
            " kill all apps at once."
        )
        sys.exit()

    df = get_list()

    all_ids = df["App ID"].unique()
    if id is not None:
        if id in all_ids:
            click.echo(f"Killing {id}...")
            os.system(f"kill -9 {id}")
        else:
            error(
                f"This app ID is not valid. Valid IDs are {all_ids}. Use `st"
                " list` to learn more."
            )

    if all:
        click.echo(f"Killing all apps...")
        os.system(f"kill -9 {' '.join(map(str, all_ids))}")
        click.echo(f"Succesfully killed ")



import click


def header():
    click.echo(
        click.style(
            "\n\nWelcome to streamlit-kickoff-cli (stk) 👞\n\n",
            fg="red",
            bold=True,
        ),
        nl=True,
    )


def choice(text: str):
    click.echo(
        click.style(f"\n{text}", bold=True),
        nl=True,
    )


def new_step(text: str):
    click.echo(click.style(f"➕ {text}", bold=False), nl=True)


def warning(text: str):
    click.echo(click.style(f"\n⚠️ {text}", fg="red", bold=True), nl=True)


def error(text: str):
    click.echo(click.style(f"\n💀 {text}", fg="red", bold=True), nl=True)


def success(text: str):
    click.echo(click.style(f"\n🎉 {text}\n", bold=True), nl=True)



import os

import click

from ..custom_logging import choice
from ..dev.command import _dev
from ..new.command import _new


@click.command()
@click.pass_context
def kick(ctx: click.Context,):
    """🚀 New app + dev set up NOW!"""

    choice("🚀 New app + dev set up NOW!")

    directory_name = _new(ctx)
    if directory_name:
        os.chdir(directory_name)
        _dev(ctx)



import webbrowser
from typing import List

import click

from .dev import command as dev_command
from .kick import command as kick_command
from .kill import command as kill_command
from .list import command as list_command
from .new import command as new_command

AVAILABLE_COMMANDS: List[click.Command] = [
    new_command.new,
    dev_command.dev,
    kick_command.kick,
    list_command.list,
    kill_command.kill,
]

def set_context_object(
    ctx: click.Context,
) -> click.Context:
    """Get useful environment information and store it.
    Throw exception if environment is not supported

    Args:
        ctx (click.Context): Current context

    Raises:
        e: Exception when environment is not supported

    Returns:
        click.Context: Updated context
    """
    try:
        ctx.ensure_object(dict)
        ctx.obj["BROWSER"] = webbrowser.get()
    except Exception as e:
        raise e
    return ctx

# Subclassing Click CLI to display commands in a specific order
class NaturalOrderGroup(click.Group):
    def list_commands(self, ctx: click.Context):
        return self.commands.keys()


@click.group(cls=NaturalOrderGroup)
@click.pass_context
def stk(ctx: click.Context,):
    """Welcome to stk 👞

    This is a simple CLI to help you kick off and
    maintain Streamlit projects as fast as possible!
    """

    ctx = set_context_object(ctx)

def add_commands_to_stk():
    for command in AVAILABLE_COMMANDS:
        stk.add_command(command)

add_commands_to_stk()

if __name__ == '__main__':
    stk()



import os
import subprocess
import webbrowser
from pathlib import Path

import click

from ..custom_logging import choice, error, new_step


def get_webbrowser(os="macos"):
    return webbrowser.get()

def _dev(ctx: click.Context, path: str = "streamlit_app.py"):
    choice("👩‍💻 Dev time! Let's open VS Code and your app in Chrome!")

    path_ = Path(path)
    # code_editor_command = ctx.obj["EDITOR_COMMAND"]

    if path_.is_file():

        # Open project in VS Code.
        new_step("Opening project in VS Code...")
        os.system(f"code . --new-window")
        os.system(f"code {path} --reuse-window")

        # Run app.
        new_step("Running app...")
        out = subprocess.check_output(["streamlit", "run", path])
        new_step(out)

        # Open app in browser.
        browser = ctx.obj["BROWSER"]
        click.echo(browser)
        new_step("Opening app in the browser...")
        browser.open_new_tab()

        # TODO: AppleScript to get the two windows side-by-side!

    else:
        error(
            f"Couldn't find the app script {path} in this directory. \nUse"
            " '-p' to specify a custom path"
        )


@click.command()
@click.option(
    "-p",
    "--path",
    default="streamlit_app.py",
    help="Path to the main script of the app (e.g. 'streamlit_app.py')",
    type=str,
)
@click.pass_context
def dev(ctx: click.Context, path: str):
    """👩‍💻 Dev time! Opens VS Code and your app in Chrome!"""
    _dev(ctx, path)



import io
import subprocess

import click
import pandas as pd

from ..custom_logging import choice


def get_list() -> pd.DataFrame:
    """Listen to Python apps running on port 85** and collect their PID

    Returns:
        pd.DataFrame: Dataframe with Python apps and PID
    """
    lsof = subprocess.check_output(
        "lsof -nP -iTCP -sTCP:LISTEN",
        shell=True,
        stderr=subprocess.STDOUT,
    )

    df = pd.read_fwf(io.BytesIO(lsof))
    df = df[df.COMMAND.str.lower().str.contains("python")]
    df["PORT"] = df.NAME.apply(lambda d: d.split("(LISTEN)")[0].split(":")[1])
    df = df[df.PORT.str.startswith("85")]
    df["URL"] = "http://localhost:" + df.PORT
    df = df.rename(columns={"PID": "App ID", "URL": "App URL"})
    return df


@click.command()
@click.pass_context
def list(ctx: click.Context,):
    """🤯 List running Streamlit apps under ports 85**"""

    choice("Let's look at your apps running locally...")

    df = get_list()
    if df.empty:
        click.echo("Found no app running!")
    else:
        click.echo(
            df[["App ID", "App URL"]].set_index("App ID").drop_duplicates()
        )



import click
from cookiecutter.main import cookiecutter

from ..custom_logging import choice, success

DEFAULT_REPOSITORY = (
    "https://github.com/arnaudmiribel/st-template/blob/main/streamlit_app.py"
)

COOKIECUTTER_DEFAULT_DIRECTORY = "app"
COOKIECUTTER_REPOSITORY = (
    "https://github.com/arnaudmiribel/st-cookiecutter.git"
)

def _new(ctx: click.Context):
    choice("🆕 You just asked for a `new` Streamlit project. Let's go!")

    directory_name = click.prompt(
        "New directory name",
        type=str,
        default=COOKIECUTTER_DEFAULT_DIRECTORY,
    )

    if click.confirm("Want to customize the template?", default=False):
        app_title = click.prompt(
            "Title of your app", type=str, default="Balloons"
        )
        app_uses_secrets = click.confirm(
            "Will you be using secrets?", default=True
        )
        app_is_multi_page = click.confirm(
            "Will your app use more than one page?", default=True
        )
        app_uses_snowflake = click.confirm(
            "Are you connecting to Snowflake in this app?", default=False
        )

        cookiecutter(
            COOKIECUTTER_REPOSITORY,
            no_input=True,
            extra_context={
                "app_name": directory_name,
                "app_title": app_title,
                "app_uses_secrets": int(app_uses_secrets),
                "app_is_multi_page": int(app_is_multi_page),
                "app_uses_snowflake": int(app_uses_snowflake),
            },
        )

    else:
        click.echo(
            "Fine! We will use default values and create your new Streamlit"
            f" app in '{directory_name}'"
        )
        cookiecutter(
            COOKIECUTTER_REPOSITORY,
            no_input=True,
            extra_context={
                "app_name": directory_name,
            },
        )

    success(
        "Successfully created your new Streamlit app in directory"
        f" '{directory_name}/'!"
    )

    return directory_name


@click.command()
@click.pass_context
def new(ctx: click.Context,):
    """🆕 Create a new Streamlit project"""
    return _new(ctx)
